class MetricsBackend(object):
    def connect(self, **connection_kwargs):
        return

    def increment(self, metric, **extra):
        raise NotImplemented

    def gauge(self, metric, value, **extra):
        raise NotImplemented

    def histogram(self, metric, value, **extra):
        raise NotImplemented

    def timed(self, metric, duration, **extra):
        raise NotImplemented



from metrics.logger import logger
import os

try:
    from dotenv import find_dotenv, load_dotenv
    load_dotenv(find_dotenv())
except ImportError:
    logger.info('no dotenv library')


DD_API_KEY = os.environ.get('DD_API_KEY')
DD_APP_KEY = os.environ.get('DD_APP_KEY')
DD_SERVICE_NAME = os.environ.get('DD_SERVICE_NAME')
DD_SERVICE_PRIORITY = os.environ.get('DD_SERVICE_PRIORITY')
DD_STATSD_SOCKET_PATH = os.environ.get('DD_STATSD_SOCKET_PATH', '/var/run/datadog/dsd.socket')
try:
    DD_GLOBAL_TAGS = dict([v.split(':') for v in os.environ.get('DD_GLOBAL_TAGS', '').split(',')])
except ValueError:
    DD_GLOBAL_TAGS = {}

DEFAULT_BACKEND = os.environ.get('METRICS_DEFAULT_BACKEND', 'metrics.backends.DataDogMetricsBackend')



from .base import MetricsBackend
from .dd import DataDogMetricsBackend
from metrics.exceptions import InvalidBackendException
from metrics import config
import inspect


def backend_factory(backend_kls):
    if backend_kls is None:
        raise ValueError('the passed backend must be a Class')

    if not issubclass(backend_kls, MetricsBackend):
        raise InvalidBackendException('backend {} is not a valid MetricsBackend as it doesnt extend MetricsBackend'.format(backend_kls))

    # we introspect on the backend constructors' kwargs to know which config vars to use as params
    try:
        spec = inspect.getfullargspec(backend_kls.__init__)
    except AttributeError:
        # if we're in a 2.x env
        spec = inspect.getargspec(backend_kls.__init__)

    spec_args = spec.args

    backend_kwargs = {}
    backend_prefix_map = {
        'DataDogMetricsBackend': 'DD'
    }

    for arg in spec_args:
        if arg == 'self':
            continue

        backend_kwargs[arg] = getattr(config, '{}_{}'.format(backend_prefix_map[backend_kls.__name__], arg.upper()))

    return backend_kls(**backend_kwargs)



class BackendConnectionException(Exception):
    """ thrown when connection to a stats backend fails. This can either be due to connectivity issues, or due to a
     misconfiguration (like missing keys)
    """
    pass


class InvalidBackendException(Exception):
    """ thrown when trying to instantiate an invalid MetricsBackend (for instance, if it isn't a subclass of MetricsBackend """
    pass


from contextlib import contextmanager
from metrics import metrics_backend
from metrics.logger import logger
import time


@contextmanager
def timer(backend=None, metric=None, **extra):
    """ timer context manager """
    _backend = backend or metrics_backend

    start_time = time.time()

    try:
        yield
    finally:
        end_time = time.time()

    delta = end_time - start_time

    logger.debug('{} -- {}'.format(metric, delta))

    _backend.timed(metric, delta, **extra)



from metrics.backends.base import MetricsBackend
from metrics import config, exceptions
from metrics.logger import logger
from datadog import initialize, statsd
from functools import wraps
import time
import os


def requires_connection(fn):
    @wraps(fn)
    def _inner(self, *args, **kwargs):
        if not self._connected:
            self.connect()

            # if we are still not connected, it means the connection credentials weren't loaded in the env
            if not self._connected:
                raise exceptions.BackendConnectionException('you must call dd.connect() with connection_kwargs')

        result = fn(self, *args, **kwargs)

        return result

    return _inner


class DataDogMetricsBackend(MetricsBackend):
    def __init__(self, service_name=None, service_priority=None, global_tags=None):
        """
        :param service_name: `str` the name of the service being monitored
        :param service_priority: `str` the service priority, on a scale of 1 to 5, 1 being the highest priority
        :param global_tags: `dict` of additional global tags to apply to all stats sent to DD
        """
        super(DataDogMetricsBackend, self).__init__()

        self.service_name = service_name
        self.service_priority = service_priority
        self.global_tags = global_tags or {}

        self._connected = False

    def connect(self, **connection_kwargs):
        options = {
            'api_key': connection_kwargs.pop('api_key', config.DD_API_KEY),
            'app_key': connection_kwargs.pop('app_key', config.DD_APP_KEY),
            'statsd_socket_path': connection_kwargs.pop('statsd_socket_path', config.DD_STATSD_SOCKET_PATH),
        }

        if not options['api_key'] or not options['app_key']:
            raise exceptions.BackendConnectionException('the DataDog backend requires keys to either be set in the env, or passed to the `connect` method')

        initialize(**options)

        self._connected = True

    @requires_connection
    def increment(self, metric, **extra):
        extra_tags = extra.pop('tags', [])

        logger.debug('sending increment metric {}'.format(metric))

        statsd.increment(metric, tags=self.statsd_tags + extra_tags)

        return True

    @requires_connection
    def gauge(self, metric, value, **extra):
        extra_tags = extra.pop('tags', [])

        logger.debug('sending gauge metric {} -- {}'.format(metric, value))

        statsd.gauge(metric, value, tags=self.statsd_tags + extra_tags)

        return True

    @requires_connection
    def histogram(self, metric, value, **extra):
        extra_tags = extra.pop('tags', [])

        logger.debug('sending histogram metric {} -- {}'.format(metric, value))

        statsd.histogram(metric, value, tags=self.statsd_tags + extra_tags)

        return True

    @requires_connection
    def timed(self, metric, duration, **extra):
        logger.debug('sending timer metric {} -- {}'.format(metric, duration))

        self.histogram(metric, duration, **extra)

        return True

    @property
    def statsd_tags(self):
        """ the global statsd tags to apply to all statsd functions """
        tag_map = {
            'service': self.service_name,
            'priority': self.service_priority,
        }

        tag_map.update(self.global_tags)

        return [':'.join([tag, val]) for tag, val in tag_map.items() if val is not None]



from functools import wraps
from metrics import metrics_backend
from metrics import context


def increment(backend=None, on_start_metric=None, on_complete_metric=None, on_error_metric=None, **extra):
    _backend = backend or metrics_backend

    def _inner(fn):
        @wraps(fn)
        def _decorator(*args, **kwargs):
            if on_start_metric:
                _backend.increment(on_start_metric, **extra)

            try:
                result = fn(*args, **kwargs)
            except Exception as exc:
                if on_error_metric:
                    _backend.increment(on_error_metric, **extra)

                raise exc

            if on_complete_metric:
                _backend.increment(on_complete_metric, **extra)

            return result

        return _decorator

    return _inner


def timer(backend=None, metric=None, **extra):
    _backend = backend or metrics_backend

    def _inner(fn):
        @wraps(fn)
        def _decorator(*args, **kwargs):
            with context.timer(backend=_backend, metric=metric, **extra):
                return fn(*args, **kwargs)

        return _decorator

    return _inner



import os
import re
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath for dirpath, dirnames, filenames in os.walk(package) if os.path.exists(os.path.join(dirpath, '__init__.py'))]


version = '0.6.0'

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='mm_metrics',
    version=version,
    url='https://github.com/Morgan-and-Morgan/metrics-python/',
    packages=get_packages('metrics'),
    include_package_data=True,
    license='BSD License',
    description='Python client library abstracting away metrics collection',
    long_description=README,
    install_requires=[
        'datadog>=0.23,<1'
    ],
    author='Morgan & Morgan Developers',
    author_email='developers@forthepeople.com',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
